/*
 * error.c --  error handling routines.
 *
 * This file is part of
 * 
 * rmdp -- 
 * 
 * (C) 1998 Lorenzo Vicisano
 *     (vicisano@cs.ucl.ac.uk)
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Luigi Rizzo,
 *      Lorenzo Vicisano and other contributors.
 * 4. Neither the name of the Authors nor the names of other contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include <stdlib.h>
#ifndef NULL
#define NULL ((void *)0)
#endif


#include <rlcif.h>
#include "rmdpif.h"

static char *error_msgs[] = {
    "* rmdp Error: library init error",
    "* rmdp Error: cannot determine file size",
    "* rmdp Error: address is not in class D",
    "* rmdp Error: cannot compute transmission param. for this file size",
    "* rmdp Error: too mamy protocol instances",
    "* rmdp Error: no such instance",
    "* rmdp Error: cannot allocate memory",
    "* rmdp Error: someting wrong with file size",
    "* rmdp Error: don't have ui stuff",
    "* rmdp Error: can't set continuation request timeout",
    "* rmdp Error: reception timeout",
    "* rmdp Error: can't send continuation request",
    "* rmdp Error: can't setup request socket",
    "* rmdp Error: bad file descriptor",
    "* rmdp Error: cannot instantiate coder instance",
    "* rmdp Error: decoder error",
    "* rmdp Error: no rlc instance id set"
};

static char *warn_msgs[] = {
    "* rmdp Warning: no error messages",
    "* rmdp Warning: cannot read request",
    "* rmdp Warning: bad request message",
    "* rmdp Warning: reception timeout",
    "* rmdp Warning: cannot write stats"
};

int rmdp_error_init_()
{
    if (rlc_register_error_table (error_msgs, RMDP_BASE_ERROR, RMDP_ERROR_SIZE)
	< 0)
	return (rlc_internal_set_error_(-1, RMDP_NOERR_H, NULL));
    if (rlc_register_error_table (warn_msgs, RMDP_BASE_WARNING,
	RMDP_WARNING_SIZE) < 0)
	return (rlc_internal_set_error_(-1, RMDP_NOERR_H, NULL));
    return (0);
}
