#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/stat.h>
#include <fcntl.h>
#if defined(Linux)                                                              
#include <unistd.h>                                                             
#endif                                                                          

#include <rlcif.h>
#include "rmdpif.h"

void
badcmdline(char *arg)
{
    rmdp_my_error(-1,
	"Bad argument %s\nCommand line:\n\t rec-sample [options]\n"
	"options:\n"
	"\t-f <output file name>\n"
	"\t-d <M-cast data address/port>\n"
	"\t-t <sender ttl>\n"
	"\t-i <protocol instance id>\n"
	"\t-C <session name>\n"
	"\t-x <enable save-file window>\n"
	"\t-g <window geometry>\n"
	"\t-e <event-log file>\n"
	"\t-p <poll-log file>\n"
	/* "\t-n <number of layers>\n" */
	,
	arg);
}

int open_n_check(char *filn)
{
  int fd;

    if  (!strcmp(filn, "-"))
	return 1;

    if  ((fd = open (filn, O_WRONLY|O_CREAT|O_TRUNC, 0644)) < 0) {
	return (-1);
    } else {
	return (fd);
    }
}

int rlc_i;		/* rlc instance id */
int rmdp_i;		/* rmdp instance id */
int fd = 1;		/* output file */
int chosefile = 0;
char *sessname=NULL;
char *geometry=NULL;


/* this handle the termination of
 * the current reception */
void reception_end(int rmdp_i, int success)
{
    if (success >= 0 && chosefile) {
#ifdef HAVE_UI
	char *ret = NULL;
	int res;
	if (sessname)
	    res = rlc_gui_tcl_exec (rlc_i, &ret,
		"tk_getSaveFile -title {%s}", sessname);
	else
	    res = rlc_gui_tcl_exec (rlc_i, &ret,
	    	"tk_getSaveFile -title \"RMDP\"");
	if (res >= 0 && ret != NULL) {
	    if (ret[0] != '\0') {
		if ((fd = open_n_check(ret)) >= 0) {
		    rmdp_set_io(rmdp_i, fd);
		} else
		    rmdp_my_error (rmdp_i,
			"Error opening out file (%s)\n", ret);
	    } else {
		rmdp_my_error (rmdp_i, "No file selected!\n");
	    }
	    free(ret);
	} else {
	    rmdp_my_error (rmdp_i, "rlc gui error|\n");
	}
#endif
    } else if (success < 0) {
	rmdp_my_error (rmdp_i, "Decoding error ? \n");
    }
    /* in this example just abort the reception instance */
    rlc_abort(rlc_i);
}

int
main(int argc, char *argv[])
{

  char dataaddr[256];
  int ttl = 5;
  int haveaddr = 0;
  int isreceiver = 1;

  int ret;

  char *ev_f = NULL , *po_f = NULL ;
  char tmpaddr[256];
  unsigned long addr0;
  short port;
  int pkt_size;		/* rmdp payload + rmdp header */
  int reqsize;
  int pid = -1;

  addr_list_t taddr;
  rmdp_addr_t adds;

  int errflg = 0, i;

  rmdp_rx_packet_t rx_f;



  extern char *optarg;
  extern int optind;

    /* parse command line... */
    while ((i = getopt(argc,argv,"g:i:t:d:f:C:e:p:x")) != EOF)
	switch(i) {
	case 'd': /* data addr/port */
	    strncpy(dataaddr, optarg, 256);
	    haveaddr = 1;
	    break;
	case 't': /* TTL */
	    ttl = atoi(optarg);
	    if (ttl <= 0 || ttl > 255) badcmdline(optarg);
	    break;
	case 'i': /* pid */
	    pid = atoi(optarg);
	    break;
	case 'f': /* output file */
	    if ((fd = open_n_check(optarg)) <= 0)
		badcmdline(optarg);
	    break;
	case 'C': /* session name */
	    if ((sessname = malloc(strlen(optarg)+1)) == NULL)
		break;
	    strcpy (sessname, optarg);
fprintf(stderr, "%s\n", sessname);
	    break;
	case 'g': /* session name */
	    if ((geometry = malloc(strlen(optarg)+1)) == NULL)
		break;
	    strcpy (geometry, optarg);
	    break;
	case 'x': /* session name */
	    chosefile = 1;
	    break;
	case 'e': /* event log file */
	    ev_f = (char *)calloc(strlen(optarg)+1, sizeof(char));
	    strncpy(ev_f, optarg, strlen(optarg)+1);
	    break;
	case 'p': /* poll log file */
	    po_f = (char *)calloc(strlen(optarg)+1, sizeof(char));
	    strncpy(po_f, optarg, strlen(optarg)+1);
	    break;
	case '?':
	    errflg++;
	}

    if (errflg)
	badcmdline("?");

    argc -= optind;
    argv += optind;

    if (!haveaddr)
	rmdp_my_error(-1, "must provide destination address");

    /* extract address (string) and port (short) from the
     * input parameter (x.x.x.x/yyy) */
    if (rmdp_parse_addr(dataaddr, tmpaddr, sizeof(tmpaddr), &port, "data") < 0)
    {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1, "Error %d in rmdp_parse_addr (%s)\n", ret, c);
    }
    addr0 = ntohl(inet_addr(tmpaddr));
    adds.addr = addr0;
    adds.port = port+1;
    adds.ttl = ttl;

    /* setup an rmdp receiver instance */
    if ((ret = rmdp_i = rmdp_new (isreceiver)) < 0 ||
	(ret = rmdp_set_addr (rmdp_i, &adds)) < 0 ||
	(ret = rmdp_set_io (rmdp_i, fd)) < 0 ) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1,
	    "Error %d in protocol initialization (%s)\n", ret, c);
    }
    if (pid != -1 && (ret = rmdp_set_id(rmdp_i, pid)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1,
	    "Error %d in protocol initialization (%s)\n", ret, c);
    }

    /* set reception-end callback */
    if ((ret = rmdp_set_end_callback(rmdp_i, reception_end)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1, 
	    "Error %d in rmdp_set_end_callback (%s)\n", ret, c);
    }

    /* get data reception handler function from rmdp */
    if ((ret = rmdp_get_f(rmdp_i, (void **)(&rx_f))) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1, 
	    "Error %d in rmdp_get_f (%s)\n", ret, c);
    }

    reqsize = pkt_size;
    taddr.nadd = 1;
    taddr.addrs = &addr0;
    taddr.port = port;

    /* setup a rlc receiver instance */
    if ((ret = rlc_i = rlc_new (isreceiver)) < 0 ||
	(ret = rlc_set_addr (rlc_i, &taddr)) < 0 ||
	(ret = rlc_attach (rlc_i, rmdp_i, (void *)rx_f)) < 0 ) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_delete(rmdp_i);
	rmdp_my_error(-1, "error %d in rlc_rx_setup (%s)\n", ret, c);
    }

#ifdef HAVE_UI
    if (geometry)
	rlc_gui_geometry (rlc_i, geometry);
    if ((ret = rlc_gui_enable(rlc_i)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_delete(rmdp_i);
	rmdp_my_error(-1, "error %d in rlc_gui_enable (%s)\n", ret, c);
    }
#endif

    /* setup rlc idle watchdog */
    if ((ret = rlc_rx_set_watchdog(rlc_i, NULL, 0)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_delete(rmdp_i);
	rmdp_my_error(-1, "error %d in rlc_rx_setup (%s)\n", ret, c);
    }

    if ((ret = rmdp_set_rlc (rmdp_i, rlc_i)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1,
	    "Error %d in protocol initialization (%s)\n", ret, c);
    }
#ifdef HAVE_UI
    /* enable UI in rmdp */
    if ((ret = rmdp_enable_gui(rmdp_i)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1,
	    "Error %d in protocol initialization (%s)\n", ret, c);
    } else if (sessname != NULL &&
	(ret = rmdp_set_name (rmdp_i, sessname)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1,
	    "Error %d in protocol initialization (%s)\n", ret, c);
    }
#endif

    /* possibly setup rlc log facility */
    if (ev_f || po_f)
	(void)rlc_rx_log(rlc_i, ev_f, po_f);

    /* start the reception */
    if ((ret = rlc_start(rlc_i)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_delete(rmdp_i);
	rmdp_my_error(-1,
	    "error %d in rlc_rx_start (%s)\n", ret, c);
    }

    while (rlc_getevent(1)!=EV_TERMINATE);

    exit(0);
}
