#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <fcntl.h>
#if defined(Linux)
#include <unistd.h>
#endif

#include <rlcif.h>
#include "rmdpif.h"

void
badcmdline(char *arg)
{
    rmdp_my_error(-1,
	"Bad argument %s\nCommand line:\n\t sen-sample [options]\n"
	"options:\n"
	"\t-f <input file name>\n"
	"\t-d <M-cast data address/port>\n"
	"\t-i <protoco instance id>\n"
	"\t-l <number of layers>\n"
	"\t-t <sender ttl>\n"
	"\t-I (non stop transmission)\n", arg);
}

int open_n_check(char *filn)
{
  int fd;

    if  (!strcmp(filn, "-"))
	return 0;

    if  ((fd = open (filn, O_RDONLY)) < 0) {
	return (-1);
    } else
	return (fd);
}

#define ISSENDER 0	/* means that is sender ! */
int
main(int argc, char *argv[])
{

  int fd = -1;		/* input file */
  char dataaddr[256];
  int ttl = 5;
  int haveaddr = 0;
  int rmdp_i;		/* rmdp instance id */
  int rlc_i;		/* rlc instance id */

  int ret;

  char tmpaddr[256];
  unsigned long addr0;
  short port;
  int pkt_size;		/* rmdp payload + rmdp header */
  int layers;
  int pid = -1;

  unsigned long base_rate = 16 * 1024;
  addr_list_t taddr;
  rlc_tx_params_t parml;	/* tx parameter list ... */
  rmdp_addr_t adds;

  unsigned long p;
  int errflg = 0, i;
  int nostop = 0;

  rmdp_tx_packet_t tx_f;



  extern char *optarg;
  extern int optind;

    /* parse command line... */
    while ((i = getopt(argc,argv,"t:d:f:i:I")) != EOF) {
	switch(i) {
	case 'd': /* data addr/port */
	    strncpy(dataaddr, optarg, 256);
	    haveaddr = 1;
	    break;
	case 'I': /* protocol instance id */
	    nostop = 1;
	    break;
	case 'i': /* protocol instance id */
	    pid = atoi(optarg);
	    break;
	case 't': /* TTL */
	    ttl = atoi(optarg);
	    if (ttl <= 0 || ttl > 255) badcmdline(optarg);
	    break;
	case 'f': /* input file */
	    if ((fd = open_n_check(optarg)) < 0)
		badcmdline(optarg);
	    break;
	case '?':
	    errflg++;
	}
    }
    if (errflg)
	badcmdline("?");

    argc -= optind;
    argv += optind;

    if (fd < 0)
	rmdp_my_error(-1, "must provide input file");
    if (!haveaddr)
	rmdp_my_error(-1, "must provide destination address");

    /* extract address (string) and port (short) from the
     * input parameter (x.x.x.x/yyy) */
    if (rmdp_parse_addr(dataaddr, tmpaddr, sizeof(tmpaddr), &port, "data") < 0)
    {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1, "Error %d in rmdp_parse_addr (%s)\n", ret, c);
    }
    addr0 = ntohl(inet_addr(tmpaddr));
    adds.addr = addr0;
    adds.port = port+1;
    adds.ttl = ttl;

    /* setup an rmdp sender instance */
    if ((ret = rmdp_i = rmdp_new (ISSENDER)) < 0 ||
	(ret = rmdp_set_addr (rmdp_i, &adds)) < 0 ||
	(ret = rmdp_set_io (rmdp_i, fd)) < 0 ) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1,
	    "Error %d in protocol initialization (%s)\n", ret, c);
    }
    if (pid != -1 && (ret = rmdp_set_id(rmdp_i, pid)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1,
	    "Error %d in protocol initialization (%s)\n", ret, c);
    }

    if (nostop) {
	unsigned long left = rmdp_get_left (rmdp_i);
	if ((ret = rmdp_incr_left (rmdp_i, 
	    (unsigned long)0xffffffff - left)) < 0) {
	    char *c;
	    ret = rlc_error_info(&c);
	    rmdp_my_error (-1,
		"Error %d in protocol initialization (%s)\n", ret, c);
	}
    }
    
    /* retrieve rmdp packet size (rlc payload) */
    pkt_size = rmdp_get_psize(rmdp_i);
    /* retrieve rmdp number of layers */
    layers = rmdp_get_nlay(rmdp_i);

    /* this is the interpacket-period (us) in the upper session */
    p = 1e6 * (pkt_size) * 8 / base_rate / (1<<(layers-1));

    /* get data transmission handler function from rmdp */
    if ((ret = rmdp_get_f(rmdp_i, (void **)(&tx_f))) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_my_error (-1,
	    "Error %d in rmdp_get_f (%s)\n", ret, c);
    }

    taddr.nadd = 1;
    taddr.addrs = &addr0;
    taddr.port = port;
    taddr.ttl = ttl;

    parml.nlay = layers;
    parml.mpkts = pkt_size;
    parml.Mpkts = pkt_size;
    parml.t = p;

    /* setup a rlc sender instance */
    if ((ret = rlc_i = rlc_new(ISSENDER)) < 0 ||
	(ret = rlc_set_addr (rlc_i, &taddr)) < 0 ||
	(ret = rlc_attach (rlc_i, rmdp_i, (void *)tx_f)) < 0 ||
	(ret = rlc_tx_setup(rlc_i, &parml)) < 0 ||
	parml.t != p ) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_delete(rmdp_i);
	rmdp_my_error(-1,
	    "error %d in rlc_tx_setup (%s)\n", ret, c);
    }

    /* start the transmission */
    if ((ret = rlc_start(rlc_i)) < 0) {
	char *c;
	ret = rlc_error_info(&c);
	rmdp_delete(rmdp_i);
	rmdp_my_error(-1,
	    "error %d in rlc_tx_start (%s)\n", ret, c);
    }

    while (rlc_getevent(1)!=EV_TERMINATE);

    exit(0);
}
